SystemOrganization addCategory: #Familiar!

TestCase subclass: #FMImportCommandTest
	instanceVariableNames: 'command'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Familiar'!

!FMImportCommandTest methodsFor: 'as yet unclassified' stamp: 'AlexandreBergel 7/1/2012 16:50'!
setUp

	command := FMImportCommand new.! !

!FMImportCommandTest methodsFor: 'as yet unclassified' stamp: 'AlexandreBergel 7/1/2012 16:51'!
testIsParseable
	self assert: (command isParseable: 'foobar = FM (A : B C [D] ;)').
	"self deny: (command isParseable: 'foobar = FM ((A : B C [D] ;)')."! !

!FMImportCommandTest methodsFor: 'as yet unclassified' stamp: 'AlexandreBergel 7/2/2012 00:28'!
testParse
	| model |
	model := command modelForTextualDescription: 'foobar = FM (A : B C [D] ;)'.
	self assert: (model class == MooseModel).
	
	self assert: (model allFeatureModels size = 1).
	self assert: (model allAssignments size = 1).
	"self assert: (model allVariables) = {'foobar'}"
	! !

TestCase subclass: #FMParserTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Familiar'!

FMParserTest subclass: #FMASTBuilderTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Familiar'!

!FMASTBuilderTest methodsFor: 'hooks' stamp: 'AlexandreBergel 7/1/2012 09:30'!
parser
	^ FMASTBuilder new! !

!FMASTBuilderTest methodsFor: 'features' stamp: 'AlexandreBergel 7/1/2012 10:56'!
testBuildingChild1
	| c firstChild secondChild thirdChild |
	c := self parser child parse: 'B'.
	self assert: c class == FMFeatureMentatory.
	self assert: c id = 'B'.
! !

!FMASTBuilderTest methodsFor: 'features' stamp: 'AlexandreBergel 7/1/2012 14:33'!
testBuildingChild2Multiple
	| c firstChild secondChild thirdChild |
	c := self parser children parse: 'B C D'.
	
	self assert: (c class == Array).
	self assert: (c allSatisfy: [ :cc | cc class ==  FMFeatureMentatory ]).
	self assert: c first id = 'B'.
	self assert: c second id = 'C'.
	self assert: c third id = 'D'.
! !

!FMASTBuilderTest methodsFor: 'features' stamp: 'AlexandreBergel 7/1/2012 10:57'!
testBuildingChild3
	| c firstChild secondChild thirdChild first |
	c := self parser children parse: 'B'.
	self assert: c class == Array.
	self assert: c size = 1.
	
	first := c first.
	self assert: first class == FMFeatureMentatory.
	self assert: first id = 'B'.
! !

!FMASTBuilderTest methodsFor: 'constraints' stamp: 'AlexandreBergel 7/1/2012 10:08'!
testBuildingConstraint1
	| c |
	c := self parser constraint parse: 'C | A'.
	self assert: c class == FMConstraintPipe.
	self assert: c right id = 'A'.
	self assert: c left id = 'C'.! !

!FMASTBuilderTest methodsFor: 'constraints' stamp: 'AlexandreBergel 7/1/2012 10:08'!
testBuildingConstraint2
	| c |
	c := self parser constraint parse: 'C -> A'.
	self assert: c class == FMConstraintArrow.
	self assert: c right id = 'A'.
	self assert: c left id = 'C'.! !

!FMASTBuilderTest methodsFor: 'constraints' stamp: 'AlexandreBergel 7/1/2012 10:08'!
testBuildingConstraint3
	| c |
	c := self parser constraint parse: 'C <-> A'.
	self assert: c class == FMConstraintEquiv.
	self assert: c right id = 'A'.
	self assert: c left id = 'C'.! !

!FMASTBuilderTest methodsFor: 'constraints' stamp: 'AlexandreBergel 7/1/2012 10:08'!
testBuildingConstraint4
	| c |
	c := self parser constraint parse: 'C | A'.
	self assert: c class == FMConstraintPipe.
	self assert: c right id = 'A'.
	self assert: c left id = 'C'.! !

!FMASTBuilderTest methodsFor: 'programs' stamp: 'AlexandreBergel 7/1/2012 15:13'!
testBuildingExample1

	| program assigment feature |
	program := self parser program parse: 'renderers11 = FM (Renderer: Product TypeInfo Elements;
					TypeInfo: Array;
					Array: CalendarEvent;
					Elements: Text Title;
					Product: CalendarRenderer;
					CalendarRenderer: RenderCalendarDetail;)'.
					
	self assert: (program class == FMProgram).
	self assert: (program numberOfStatements = 1).

	assigment := program statementAt: 1.
	self assert: (assigment variable = 'renderers11'). 
	
	feature := assigment featureModel.
	self assert: ((feature productionAt: 1) root = 'Renderer').
	self assert: ((feature productionAt: 1) numberOfChildren = 3).
	
	self assert: (feature numberOfConstraints isZero).
	self assert: (feature numberOfProductions = 6).! !

!FMASTBuilderTest methodsFor: 'features' stamp: 'AlexandreBergel 7/1/2012 14:51'!
testBuildingFeature1
	| c prod |
	c := self parser featureModel parse: 'FM ( A : B ; )'.
	self assert: c class == FMFeatureModel.
	self assert: c numberOfProductions = 1.
	self assert: c numberOfConstraints  isZero.
	
	prod := (c productionAt: 1).
	self assert: prod root = 'A'.
	self assert: prod numberOfChildren = 1.
	 
	self assert: (prod childAt: 1 ) id = 'B' .! !

!FMASTBuilderTest methodsFor: 'features' stamp: 'AlexandreBergel 7/1/2012 15:25'!
testBuildingFeature2
	| c prod |
	c := self parser featureModel parse: 'FM (Renderer: Product TypeInfo Elements;
					TypeInfo: Array;
					Array: CalendarEvent;
					Elements: Text Title;
					Product: CalendarRenderer;
					CalendarRenderer: RenderCalendarDetail;)'.
	self assert: c class == FMFeatureModel.
	self assert: c numberOfProductions = 6.
	self assert: c numberOfConstraints  isZero.
	
	prod := (c productionAt: 1).
	self assert: prod root =  'Renderer'.
	self assert: prod numberOfChildren = 3.
	 
	prod := (c productionAt: 6).
	self assert: prod root =  'CalendarRenderer'.
	self assert: prod numberOfChildren = 1.
	self assert: (prod childAt: 1) id = 'RenderCalendarDetail'.	! !

!FMASTBuilderTest methodsFor: 'constraints' stamp: 'AlexandreBergel 7/1/2012 10:08'!
testBuildingNegated
	| c |
	c := self parser constraint parse: '!! A'.
	self assert: c class == FMConstraintNegated.
	self assert: c expr id = 'A'! !

!FMASTBuilderTest methodsFor: 'features' stamp: 'AlexandreBergel 7/1/2012 10:35'!
testBuildingProduction1
	| c firstChild |
	c := self parser production parse: 'A : B'.
	self assert: c class == FMFeatureProduction.
	self assert: c root = 'A'.
	self assert: c numberOfChildren = 1.

	firstChild := c childAt: 1.
	self assert: firstChild class ==  FMFeatureMentatory.! !

!FMASTBuilderTest methodsFor: 'features' stamp: 'AlexandreBergel 7/1/2012 10:43'!
testBuildingProduction2
	| c firstChild secondChild thirdChild |
	c := self parser production parse: 'A : B C D'.
	self assert: c class == FMFeatureProduction.
	self assert: c root = 'A'.
	self assert: c numberOfChildren = 3.

	firstChild := c childAt: 1.
	secondChild := c childAt: 2.
	thirdChild := c childAt: 3.
	
	self assert: firstChild id = 'B'.
	self assert: secondChild class ==  FMFeatureMentatory.
	self assert: thirdChild class ==  FMFeatureMentatory.
	self assert: secondChild id = 'C'.
	self assert: thirdChild id = 'D'.
	! !

!FMASTBuilderTest methodsFor: 'programs' stamp: 'AlexandreBergel 7/1/2012 16:30'!
testProgram
	| program |
	program := FMASTBuilder new program parse: 
'Renderers1 = FM (Renderer: Product TypeInfo Elements;
					TypeInfo: Array;
					Array: Picture;
					Elements: Graphic;
					Graphic: Full;
					Product: AlbumRenderer;
					AlbumRenderer: MosaicAlbum;)
					
renderers2 = FM (Renderer: Product TypeInfo Elements;
					TypeInfo: Array;
					Array: Picture;
					Elements: Graphic;
					Graphic: Thumbnails;
					Product: AlbumRenderer;
					AlbumRenderer: ThumbnailsPicture;)
					
renderers3 = FM (Renderer: Product TypeInfo Elements;
					TypeInfo: Array;
					Array: News;
					Elements: Text [Graphic] Title Logo;
					Graphic: Full;
					Product: NewsRenderer;
					NewsRenderer: RenderNewsDetails;)
					
renderers4 = FM (Renderer: Product TypeInfo Elements;
					TypeInfo: Array;
					Array: News;
					Elements: Text;
					Product: NewsRenderer;
					NewsRenderer: RenderNewsTitle;)	
					
renderers5 = FM (Renderer: Product TypeInfo Elements;
					TypeInfo: Array;
					Array: Tweet;
					Elements: Text;
					Product: TwitterRenderer;
					TwitterRenderer: RenderTweetContent;)
					
renderers6 = FM (Renderer: Product TypeInfo Elements;
					TypeInfo: Array;
					Array: Tweet;
					Elements: Text Graphic;
					Graphic: Thumbnails;
					Product: TwitterRenderer;
					TwitterRenderer: RenderTweetFull;)
					
renderers7 = FM (Renderer: Product TypeInfo Elements;
					TypeInfo: Array;
					Array: Picture;
					Elements: Graphic;
					Graphic: Full;
					Product: AlbumRenderer;
					AlbumRenderer: FullscreenPicture;)
					
renderers8 = FM (Renderer: Product TypeInfo Elements;
					TypeInfo: Array;
					Array: Annonce;
					Elements: Text;
					Product: AnnounceRenderer;
					AnnounceRenderer: RenderAnnonceTitle;)
					
renderers9 = FM (Renderer: Product TypeInfo Elements;
					TypeInfo: Array;
					Array: Annonce;
					Elements: Text Title Logo;
					Product: AnnounceRenderer;
					AnnounceRenderer: RenderAnnonceDetail;)	
					
renderers10 = FM (Renderer: Product TypeInfo Elements;
					TypeInfo: Array;
					Array: Annonce;
					Elements: Text Title Logo;
					Product: AnnounceRenderer;
					AnnounceRenderer: RenderAnnonceDetail;)	
					
renderers11 = FM (Renderer: Product TypeInfo Elements;
					TypeInfo: Array;
					Array: CalendarEvent;
					Elements: Text Title;
					Product: CalendarRenderer;
					CalendarRenderer: RenderCalendarDetail;)
					
renderers12 = FM (Renderer: Product TypeInfo Elements;
					TypeInfo: Array;
					Array: CalendarEvent;
					Elements: Text;
					Product: CalendarRenderer;
					CalendarRenderer: RenderCalendarTitle;)
		
					
rendererMerge = FM(Renderer: TypeInfo Product Elements ; 
TypeInfo: Array ; 
Product: (TwitterRenderer|AnnounceRenderer|CalendarRenderer|AlbumRenderer|NewsRenderer) ; 
Elements: (Graphic|Text)+ [Title] [Logo] ; 
Array: (Tweet|Annonce|CalendarEvent|Picture|News) ; 
TwitterRenderer: (RenderTweetContent|RenderTweetFull) ; 
AnnounceRenderer: (RenderAnnonceTitle|RenderAnnonceDetail) ; 
CalendarRenderer: (RenderCalendarDetail|RenderCalendarTitle) ; 
AlbumRenderer: (ThumbnailsPicture|FullscreenPicture) ; 
NewsRenderer: (RenderNewsDetails|RenderNewsTitle) ; 
Graphic: (Thumbnails|Full) ; 
(RenderAnnonceTitle -> Text);
(RenderAnnonceDetail -> !!Graphic);
(RenderTweetFull -> !!Logo);
(Thumbnails -> !!CalendarRenderer);
(CalendarEvent -> Text);
(Picture -> Graphic);
(RenderNewsDetails -> Text);
(Text -> !!FullscreenPicture);
(Annonce -> Text);
(RenderTweetFull -> Thumbnails);
(Title -> !!RenderNewsTitle);
(FullscreenPicture -> Full);
(Full -> !!RenderNewsTitle);
(Title -> !!Tweet);
(TwitterRenderer -> !!Full);
(CalendarRenderer -> !!Logo);
(RenderNewsTitle -> Text);
(Text -> !!Picture);
(RenderAnnonceTitle -> !!Graphic);
(CalendarEvent -> !!Full);
(Full -> !!AnnounceRenderer);
(Thumbnails -> !!RenderNewsDetails);
(Thumbnails -> !!RenderNewsTitle);
(RenderAnnonceDetail -> Title);
(Tweet <-> TwitterRenderer);
(NewsRenderer -> Text);
(RenderNewsDetails -> Logo);
(RenderAnnonceDetail -> Logo);
(CalendarRenderer <-> CalendarEvent);
(Thumbnails -> !!News);
(Picture <-> AlbumRenderer);
(Thumbnails -> !!RenderTweetContent);
(RenderCalendarDetail -> Title);
(Graphic -> !!RenderTweetContent);
(FullscreenPicture -> Graphic);
(Thumbnails -> !!Annonce);
(ThumbnailsPicture -> Thumbnails);
(Graphic -> !!RenderCalendarTitle);
(TwitterRenderer -> Text);
(RenderTweetFull -> Graphic);
(RenderAnnonceTitle -> !!Title);
(RenderNewsDetails -> Title);
(Title -> !!Thumbnails);
(CalendarRenderer -> !!Graphic);
(Title -> !!RenderCalendarTitle);
(Logo -> Title);
(RenderNewsTitle -> !!Graphic);
(NewsRenderer <-> News);
(Logo -> Text);
(AnnounceRenderer -> !!Graphic);
(Title -> Text);
(Annonce <-> AnnounceRenderer);)
'.

	self assert: program numberOfStatements = 13.
	self assert: program lastStatement featureModel numberOfProductions = 11.
	self assert: program lastStatement featureModel numberOfConstraints = 52.! !

!FMParserTest methodsFor: 'examples' stamp: 'AlexandreBergel 7/1/2012 08:45'!
parseFeatureModel: str
	self assert: (FMParser new featureModel matches: str).
	! !

!FMParserTest methodsFor: 'examples' stamp: 'AlexandreBergel 7/1/2012 08:48'!
parseProgram: str
	self assert: (FMParser new program matches: str).
	! !

!FMParserTest methodsFor: 'examples' stamp: 'AlexandreBergel 7/1/2012 08:47'!
parseStatement: str
	self assert: (FMParser new statement matches: str).
	! !

!FMParserTest methodsFor: 'hooks' stamp: 'AlexandreBergel 7/1/2012 09:24'!
parser
	^ FMParser new! !

!FMParserTest methodsFor: 'tests' stamp: 'AlexandreBergel 7/1/2012 09:24'!
testConstraint1
	self assert: (self parser constraint matches: 'C | A').! !

!FMParserTest methodsFor: 'tests' stamp: 'AlexandreBergel 7/1/2012 09:25'!
testConstraint2
	self assert: (self parser constraint matches: '!! C | A').! !

!FMParserTest methodsFor: 'tests' stamp: 'AlexandreBergel 7/1/2012 09:25'!
testConstraint3
	self assert: (self parser constraint matches: '(C | A)').! !

!FMParserTest methodsFor: 'tests' stamp: 'AlexandreBergel 7/1/2012 09:25'!
testExample
	self assert: (self parser featureModel matches: 'FM (A : B C [D]; D : (E|F|G); C : (H|I|J)+ ;  C | I ; )')! !

!FMParserTest methodsFor: 'tests' stamp: 'AlexandreBergel 7/1/2012 09:25'!
testExample10WithConstraint

	self assert: (self parser featureModel matches: 'FM (A : B ;  (RenderAnnonceTitle -> Text); (RenderAnnonceTitle -> Text);)')! !

!FMParserTest methodsFor: 'tests' stamp: 'AlexandreBergel 7/1/2012 09:25'!
testExample11WithConstraint

	self assert: (self parser featureModel matches: 'FM (A : B ;  (RenderAnnonceTitle -> Text); (Thumbnails -> !!RenderNewsTitle); )')! !

!FMParserTest methodsFor: 'tests' stamp: 'AlexandreBergel 7/1/2012 09:25'!
testExample2
	self assert: (self parser featureModel matches: 'FM (A : B C [ D ] ; )')! !

!FMParserTest methodsFor: 'tests' stamp: 'AlexandreBergel 7/1/2012 09:25'!
testExample3
	self assert: (self parser featureModel matches: 'FM ( A : B  ; )')! !

!FMParserTest methodsFor: 'tests' stamp: 'AlexandreBergel 7/1/2012 09:25'!
testExample4
	self assert: (self parser featureModel matches: 'FM (A : B  C  ; )')! !

!FMParserTest methodsFor: 'tests' stamp: 'AlexandreBergel 7/1/2012 09:25'!
testExample5
	self assert: (self parser featureModel matches: 'FM (A :  (B  | C); )')! !

!FMParserTest methodsFor: 'tests' stamp: 'AlexandreBergel 7/1/2012 09:25'!
testExample6
	self assert: (self parser featureModel matches: 'FM (A :  (B  | C); A :  (B  | C);  )')! !

!FMParserTest methodsFor: 'tests' stamp: 'AlexandreBergel 7/1/2012 09:25'!
testExample7
	self assert: (self parser featureModel matches: 'FM ( C : (H|I|J) + ;  )')! !

!FMParserTest methodsFor: 'tests' stamp: 'AlexandreBergel 7/1/2012 09:25'!
testExample8WithConstraint
	self assert: (self parser featureModel matches: 'FM (A : B C [D]; D : (E|F|G); C : (H|I|J)+ ;  !! C | I ; )')! !

!FMParserTest methodsFor: 'tests' stamp: 'AlexandreBergel 7/1/2012 09:26'!
testExample9WithConstraint

	self assert: (self parser featureModel matches: 'FM (A : B ;  (RenderAnnonceTitle -> Text); )')! !

!FMParserTest methodsFor: 'examples' stamp: 'AlexandreBergel 7/1/2012 08:47'!
testExamples

	self parseStatement: 'Renderers1 = FM (Renderer: Product TypeInfo Elements;
					TypeInfo: Array;
					Array: Picture;
					Elements: Graphic;
					Graphic: Full;
					Product: AlbumRenderer;
					AlbumRenderer: MosaicAlbum;)'! !

!FMParserTest methodsFor: 'tests' stamp: 'AlexandreBergel 7/1/2012 09:26'!
testIdentifier

	self assert: (self parser fmlIdentifier parse: 'HelloWorld') = 'HelloWorld'! !

!FMParserTest methodsFor: 'tests' stamp: 'AlexandreBergel 7/1/2012 14:12'!
testIdentifier2

	self assert: (self parser fmlIdentifier parse: 'Hello World') = 'Hello' ! !

!FMParserTest methodsFor: 'examples' stamp: 'AlexandreBergel 7/1/2012 10:24'!
testParsing

	| ast |
	ast := self parser program parse: 'Renderers1 = FM (Renderer: Product TypeInfo Elements;
					TypeInfo: Array;
					Array: Picture;
					Elements: Graphic;
					Graphic: Full;
					Product: AlbumRenderer;
					AlbumRenderer: MosaicAlbum;)'.
					
	! !

!FMParserTest methodsFor: 'tests' stamp: 'AlexandreBergel 7/1/2012 09:26'!
testProduction
	self assert: (self parser production matches: 'A : B ')! !

!FMParserTest methodsFor: 'examples' stamp: 'AlexandreBergel 7/1/2012 09:12'!
testProgram
	self parseProgram: 'Renderers1 = FM (Renderer: Product TypeInfo Elements;
					TypeInfo: Array;
					Array: Picture;
					Elements: Graphic;
					Graphic: Full;
					Product: AlbumRenderer;
					AlbumRenderer: MosaicAlbum;)
					
renderers2 = FM (Renderer: Product TypeInfo Elements;
					TypeInfo: Array;
					Array: Picture;
					Elements: Graphic;
					Graphic: Thumbnails;
					Product: AlbumRenderer;
					AlbumRenderer: ThumbnailsPicture;)
					
renderers3 = FM (Renderer: Product TypeInfo Elements;
					TypeInfo: Array;
					Array: News;
					Elements: Text [Graphic] Title Logo;
					Graphic: Full;
					Product: NewsRenderer;
					NewsRenderer: RenderNewsDetails;)
					
renderers4 = FM (Renderer: Product TypeInfo Elements;
					TypeInfo: Array;
					Array: News;
					Elements: Text;
					Product: NewsRenderer;
					NewsRenderer: RenderNewsTitle;)	
					
renderers5 = FM (Renderer: Product TypeInfo Elements;
					TypeInfo: Array;
					Array: Tweet;
					Elements: Text;
					Product: TwitterRenderer;
					TwitterRenderer: RenderTweetContent;)
					
renderers6 = FM (Renderer: Product TypeInfo Elements;
					TypeInfo: Array;
					Array: Tweet;
					Elements: Text Graphic;
					Graphic: Thumbnails;
					Product: TwitterRenderer;
					TwitterRenderer: RenderTweetFull;)
					
renderers7 = FM (Renderer: Product TypeInfo Elements;
					TypeInfo: Array;
					Array: Picture;
					Elements: Graphic;
					Graphic: Full;
					Product: AlbumRenderer;
					AlbumRenderer: FullscreenPicture;)
					
renderers8 = FM (Renderer: Product TypeInfo Elements;
					TypeInfo: Array;
					Array: Annonce;
					Elements: Text;
					Product: AnnounceRenderer;
					AnnounceRenderer: RenderAnnonceTitle;)
					
renderers9 = FM (Renderer: Product TypeInfo Elements;
					TypeInfo: Array;
					Array: Annonce;
					Elements: Text Title Logo;
					Product: AnnounceRenderer;
					AnnounceRenderer: RenderAnnonceDetail;)	
					
renderers10 = FM (Renderer: Product TypeInfo Elements;
					TypeInfo: Array;
					Array: Annonce;
					Elements: Text Title Logo;
					Product: AnnounceRenderer;
					AnnounceRenderer: RenderAnnonceDetail;)	
					
renderers11 = FM (Renderer: Product TypeInfo Elements;
					TypeInfo: Array;
					Array: CalendarEvent;
					Elements: Text Title;
					Product: CalendarRenderer;
					CalendarRenderer: RenderCalendarDetail;)
					
renderers12 = FM (Renderer: Product TypeInfo Elements;
					TypeInfo: Array;
					Array: CalendarEvent;
					Elements: Text;
					Product: CalendarRenderer;
					CalendarRenderer: RenderCalendarTitle;)
		
					
rendererMerge = FM(Renderer: TypeInfo Product Elements ; 
TypeInfo: Array ; 
Product: (TwitterRenderer|AnnounceRenderer|CalendarRenderer|AlbumRenderer|NewsRenderer) ; 
Elements: (Graphic|Text)+ [Title] [Logo] ; 
Array: (Tweet|Annonce|CalendarEvent|Picture|News) ; 
TwitterRenderer: (RenderTweetContent|RenderTweetFull) ; 
AnnounceRenderer: (RenderAnnonceTitle|RenderAnnonceDetail) ; 
CalendarRenderer: (RenderCalendarDetail|RenderCalendarTitle) ; 
AlbumRenderer: (ThumbnailsPicture|FullscreenPicture) ; 
NewsRenderer: (RenderNewsDetails|RenderNewsTitle) ; 
Graphic: (Thumbnails|Full) ; 
(RenderAnnonceTitle -> Text);
(RenderAnnonceDetail -> !!Graphic);
(RenderTweetFull -> !!Logo);
(Thumbnails -> !!CalendarRenderer);
(CalendarEvent -> Text);
(Picture -> Graphic);
(RenderNewsDetails -> Text);
(Text -> !!FullscreenPicture);
(Annonce -> Text);
(RenderTweetFull -> Thumbnails);
(Title -> !!RenderNewsTitle);
(FullscreenPicture -> Full);
(Full -> !!RenderNewsTitle);
(Title -> !!Tweet);
(TwitterRenderer -> !!Full);
(CalendarRenderer -> !!Logo);
(RenderNewsTitle -> Text);
(Text -> !!Picture);
(RenderAnnonceTitle -> !!Graphic);
(CalendarEvent -> !!Full);
(Full -> !!AnnounceRenderer);
(Thumbnails -> !!RenderNewsDetails);
(Thumbnails -> !!RenderNewsTitle);
(RenderAnnonceDetail -> Title);
(Tweet <-> TwitterRenderer);
(NewsRenderer -> Text);
(RenderNewsDetails -> Logo);
(RenderAnnonceDetail -> Logo);
(CalendarRenderer <-> CalendarEvent);
(Thumbnails -> !!News);
(Picture <-> AlbumRenderer);
(Thumbnails -> !!RenderTweetContent);
(RenderCalendarDetail -> Title);
(Graphic -> !!RenderTweetContent);
(FullscreenPicture -> Graphic);
(Thumbnails -> !!Annonce);
(ThumbnailsPicture -> Thumbnails);
(Graphic -> !!RenderCalendarTitle);
(TwitterRenderer -> Text);
(RenderTweetFull -> Graphic);
(RenderAnnonceTitle -> !!Title);
(RenderNewsDetails -> Title);
(Title -> !!Thumbnails);
(CalendarRenderer -> !!Graphic);
(Title -> !!RenderCalendarTitle);
(Logo -> Title);
(RenderNewsTitle -> !!Graphic);
(NewsRenderer <-> News);
(Logo -> Text);
(AnnounceRenderer -> !!Graphic);
(Title -> Text);
(Annonce <-> AnnounceRenderer);)
'! !

!FMParserTest methodsFor: 'examples' stamp: 'AlexandreBergel 7/1/2012 09:13'!
testProgram2
	self parseProgram: 'rendererMerge = FM(Renderer: TypeInfo Product Elements ; 
TypeInfo: Array ; 
Product: (TwitterRenderer|AnnounceRenderer|CalendarRenderer|AlbumRenderer|NewsRenderer) ; 
Elements: (Graphic|Text)+ [Title] [Logo] ; 
Array: (Tweet|Annonce|CalendarEvent|Picture|News) ; 
TwitterRenderer: (RenderTweetContent|RenderTweetFull) ; 
AnnounceRenderer: (RenderAnnonceTitle|RenderAnnonceDetail) ; 
CalendarRenderer: (RenderCalendarDetail|RenderCalendarTitle) ; 
AlbumRenderer: (ThumbnailsPicture|FullscreenPicture) ; 
NewsRenderer: (RenderNewsDetails|RenderNewsTitle) ; 
Graphic: (Thumbnails|Full) ; 
(RenderAnnonceTitle -> Text);
(RenderAnnonceDetail -> !!Graphic);
(RenderTweetFull -> !!Logo);
(Thumbnails -> !!CalendarRenderer);
(CalendarEvent -> Text);
(Picture -> Graphic);
(RenderNewsDetails -> Text);
(Text -> !!FullscreenPicture);
(Annonce -> Text);
(RenderTweetFull -> Thumbnails);
(Title -> !!RenderNewsTitle);
(FullscreenPicture -> Full);
(Full -> !!RenderNewsTitle);
(Title -> !!Tweet);
(TwitterRenderer -> !!Full);
(CalendarRenderer -> !!Logo);
(RenderNewsTitle -> Text);
(Text -> !!Picture);
(RenderAnnonceTitle -> !!Graphic);
(CalendarEvent -> !!Full);
(Full -> !!AnnounceRenderer);
(Thumbnails -> !!RenderNewsDetails);
(Thumbnails -> !!RenderNewsTitle);
(RenderAnnonceDetail -> Title);
(Tweet <-> TwitterRenderer);
(NewsRenderer -> Text);
(RenderNewsDetails -> Logo);
(RenderAnnonceDetail -> Logo);
(CalendarRenderer <-> CalendarEvent);
(Thumbnails -> !!News);
(Picture <-> AlbumRenderer);
(Thumbnails -> !!RenderTweetContent);
(RenderCalendarDetail -> Title);
(Graphic -> !!RenderTweetContent);
(FullscreenPicture -> Graphic);
(Thumbnails -> !!Annonce);
(ThumbnailsPicture -> Thumbnails);
(Graphic -> !!RenderCalendarTitle);
(TwitterRenderer -> Text);
(RenderTweetFull -> Graphic);
(RenderAnnonceTitle -> !!Title);
(RenderNewsDetails -> Title);
(Title -> !!Thumbnails);
(CalendarRenderer -> !!Graphic);
(Title -> !!RenderCalendarTitle);
(Logo -> Title);
(RenderNewsTitle -> !!Graphic);
(NewsRenderer <-> News);
(Logo -> Text);
(AnnounceRenderer -> !!Graphic);
(Title -> Text);
(Annonce <-> AnnounceRenderer);)
'! !

MPImportCommand subclass: #FMImportCommand
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Familiar'!

!FMImportCommand methodsFor: 'hooks' stamp: 'AlexandreBergel 7/1/2012 16:45'!
execute
	! !

!FMImportCommand methodsFor: 'util' stamp: 'AlexandreBergel 7/1/2012 16:48'!
isParseable: string
	^ FMASTBuilder new program matches: string! !

!FMImportCommand methodsFor: 'hooks' stamp: 'AlexandreBergel 7/1/2012 16:44'!
label
	^ 'Import Familiar description from textual description'! !

!FMImportCommand methodsFor: 'hooks' stamp: 'AlexandreBergel 7/2/2012 00:13'!
modelForTextualDescription: string
	"parse string and fill up a moose model. string has to correspond to a program"
	
	| model program collectingVisitor |
	
	"Make sure that the code is parseable"
	self assert: [ self isParseable: string ] description: 'Entered code is not parseable. Please check'.
	model := MooseModel new.
	
	program := FMASTBuilder new program parse: string.
	collectingVisitor := CollectingVisitor new.
	program accept: collectingVisitor.
	model addAll: collectingVisitor elements.
	
	^ model
	! !

MooseEntity subclass: #FMObject
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Familiar'!

FMObject subclass: #FMConstraint
	instanceVariableNames: 'exprs'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Familiar'!

FMObject subclass: #FMConstraintExpr
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Familiar'!

FMConstraintExpr subclass: #FMConstraintBinOp
	instanceVariableNames: 'left right'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Familiar'!

FMConstraintBinOp subclass: #FMConstraintAnd
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Familiar'!

!FMConstraintAnd class methodsFor: 'as yet unclassified' stamp: 'AlexandreBergel 7/1/2012 09:59'!
operator
	^ '&'! !

FMConstraintBinOp subclass: #FMConstraintArrow
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Familiar'!

!FMConstraintArrow class methodsFor: 'as yet unclassified' stamp: 'AlexandreBergel 7/1/2012 09:59'!
operator
	^ '->'! !

!FMConstraintBinOp class methodsFor: 'as yet unclassified' stamp: 'AlexandreBergel 7/1/2012 10:01'!
for: string
	"Return the class that correspond to the string. 
	We here assume that all the subclasses have a distint operator"
	^ self allSubclasses detect: [:cls | cls operator = string]! !

!FMConstraintBinOp class methodsFor: 'as yet unclassified' stamp: 'AlexandreBergel 7/1/2012 10:00'!
operator
	^ self subclassResponsibility ! !

!FMConstraintBinOp methodsFor: 'accessing' stamp: 'AlexandreBergel 7/1/2012 10:01'!
left
	^ left! !

!FMConstraintBinOp methodsFor: 'accessing' stamp: 'AlexandreBergel 7/1/2012 10:01'!
left: anObject
	left := anObject! !

!FMConstraintBinOp methodsFor: 'accessing' stamp: 'AlexandreBergel 7/1/2012 10:02'!
right
	^ right! !

!FMConstraintBinOp methodsFor: 'accessing' stamp: 'AlexandreBergel 7/1/2012 10:02'!
right: anObject
	right := anObject! !

FMConstraintBinOp subclass: #FMConstraintEquiv
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Familiar'!

!FMConstraintEquiv class methodsFor: 'as yet unclassified' stamp: 'AlexandreBergel 7/1/2012 09:59'!
operator
	^ '<->'! !

FMConstraintBinOp subclass: #FMConstraintPipe
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Familiar'!

!FMConstraintPipe class methodsFor: 'as yet unclassified' stamp: 'AlexandreBergel 7/1/2012 09:59'!
operator
	^ '|'! !

FMConstraintExpr subclass: #FMConstraintNegated
	instanceVariableNames: 'expr'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Familiar'!

!FMConstraintNegated methodsFor: 'accessing' stamp: 'AlexandreBergel 7/1/2012 10:05'!
expr
	^ expr! !

!FMConstraintNegated methodsFor: 'accessing' stamp: 'AlexandreBergel 7/1/2012 10:05'!
expr: anObject
	expr := anObject! !

FMConstraintExpr subclass: #FMConstraintValue
	instanceVariableNames: 'id'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Familiar'!

!FMConstraintValue methodsFor: 'accessing' stamp: 'AlexandreBergel 7/1/2012 10:07'!
id
	^ id! !

!FMConstraintValue methodsFor: 'accessing' stamp: 'AlexandreBergel 7/1/2012 10:07'!
id: anObject
	id := anObject! !

FMObject subclass: #FMFeatureExpr
	instanceVariableNames: 'id'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Familiar'!

!FMFeatureExpr methodsFor: 'visiting' stamp: 'AlexandreBergel 7/2/2012 00:01'!
accept: aVisitor
	self subclassResponsibility! !

!FMFeatureExpr methodsFor: 'accessing' stamp: 'AlexandreBergel 7/1/2012 10:39'!
id
	^ id! !

!FMFeatureExpr methodsFor: 'accessing' stamp: 'AlexandreBergel 7/1/2012 10:48'!
id: anObject
	self assert: [ anObject class == ByteString ].
	self assert: [ (anObject includes: $ ) not ].
	id := anObject! !

!FMFeatureExpr methodsFor: 'printing' stamp: 'AlexandreBergel 7/1/2012 10:45'!
printOn: str
	super printOn: str.
	str nextPut: $<.
	id ifNotNil: [ str nextPutAll: id ].
	str nextPut: $>.! !

FMFeatureExpr subclass: #FMFeatureMentatory
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Familiar'!

FMFeatureExpr subclass: #FMFeatureMutex
	instanceVariableNames: 'expr'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Familiar'!

FMFeatureExpr subclass: #FMFeatureOptional
	instanceVariableNames: 'expr'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Familiar'!

FMFeatureExpr subclass: #FMFeatureOr
	instanceVariableNames: 'expr'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Familiar'!

FMFeatureExpr subclass: #FMFeatureProduction
	instanceVariableNames: 'root children'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Familiar'!

!FMFeatureProduction methodsFor: 'accessing' stamp: 'AlexandreBergel 7/1/2012 10:32'!
addAll: featureExpressions
	self assert: [ featureExpressions isCollection ].

	children addAll: featureExpressions! !

!FMFeatureProduction methodsFor: 'accessing' stamp: 'AlexandreBergel 7/1/2012 10:37'!
childAt: index
	^ children at: index! !

!FMFeatureProduction methodsFor: 'initialize-release' stamp: 'AlexandreBergel 7/1/2012 10:32'!
initialize
	super initialize.
	children := OrderedCollection new! !

!FMFeatureProduction methodsFor: 'metrics' stamp: 'AlexandreBergel 7/1/2012 10:37'!
numberOfChildren
	^ children size! !

!FMFeatureProduction methodsFor: 'printing' stamp: 'AlexandreBergel 7/1/2012 15:23'!
printOn: str
	super printOn: str.
	str nextPut: $<.
	id ifNotNil: [ str nextPutAll: root ].
	str nextPut: $>.! !

!FMFeatureProduction methodsFor: 'accessing' stamp: 'AlexandreBergel 7/1/2012 10:31'!
root
	^ root! !

!FMFeatureProduction methodsFor: 'accessing' stamp: 'AlexandreBergel 7/1/2012 10:31'!
root: anObject
	root := anObject! !

FMFeatureExpr subclass: #FMFeatureValue
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Familiar'!

FMFeatureExpr subclass: #FMFeatureXor
	instanceVariableNames: 'expr'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Familiar'!

FMObject subclass: #FMFeatureModel
	instanceVariableNames: 'exprs constraints'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Familiar'!

!FMFeatureModel methodsFor: 'visiting' stamp: 'AlexandreBergel 7/2/2012 00:17'!
accept: aVisitor 
	aVisitor visitFeatureModel: self! !

!FMFeatureModel methodsFor: 'adding' stamp: 'AlexandreBergel 7/1/2012 14:23'!
addAllConstraints: cons
	constraints addAll: cons! !

!FMFeatureModel methodsFor: 'adding' stamp: 'AlexandreBergel 7/1/2012 15:22'!
addAllExpressions: exps
	exprs addAll: exps! !

!FMFeatureModel methodsFor: 'accessing' stamp: 'AlexandreBergel 7/1/2012 14:50'!
constraintAt: index
	^ constraints at: index! !

!FMFeatureModel methodsFor: 'initialize-release' stamp: 'AlexandreBergel 7/1/2012 14:26'!
initialize 
	super initialize.
	exprs := OrderedCollection new.
	constraints := OrderedCollection new! !

!FMFeatureModel methodsFor: 'metrics' stamp: 'AlexandreBergel 7/1/2012 14:49'!
numberOfConstraints
	^ constraints size! !

!FMFeatureModel methodsFor: 'metrics' stamp: 'AlexandreBergel 7/1/2012 14:49'!
numberOfProductions
	^ exprs size! !

!FMFeatureModel methodsFor: 'accessing' stamp: 'AlexandreBergel 7/1/2012 14:50'!
productionAt: index
	^ exprs at: index! !

FMObject subclass: #FMProgram
	instanceVariableNames: 'statements'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Familiar'!

!FMProgram methodsFor: 'visiting' stamp: 'AlexandreBergel 7/2/2012 00:03'!
accept: aVisitor
	aVisitor visitProgram: self.
	self statements do: [ :s | s accept: aVisitor ]! !

!FMProgram methodsFor: 'adding' stamp: 'AlexandreBergel 7/1/2012 14:57'!
add: aStatement
	statements add: aStatement ! !

!FMProgram methodsFor: 'adding' stamp: 'AlexandreBergel 7/1/2012 15:06'!
addAll: ss
	ss do: [ :s | self add: s ]! !

!FMProgram methodsFor: 'initialize-release' stamp: 'AlexandreBergel 7/1/2012 14:57'!
initialize
	super initialize.
	statements := OrderedCollection new! !

!FMProgram methodsFor: 'accessing' stamp: 'AlexandreBergel 7/1/2012 16:24'!
lastStatement 
	^ statements last! !

!FMProgram methodsFor: 'metrics' stamp: 'AlexandreBergel 7/1/2012 14:58'!
numberOfStatements
	^ statements size! !

!FMProgram methodsFor: 'accessing' stamp: 'AlexandreBergel 7/1/2012 15:08'!
statementAt: index
	^ statements at: index! !

!FMProgram methodsFor: 'accessing' stamp: 'AlexandreBergel 7/2/2012 00:14'!
statements
	^ statements copy! !

FMObject subclass: #FMStatement
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Familiar'!

FMStatement subclass: #FMAssignment
	instanceVariableNames: 'variable featureModel'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Familiar'!

!FMAssignment methodsFor: 'visiting' stamp: 'AlexandreBergel 7/2/2012 00:16'!
accept: aVisitor
	aVisitor visitAssigment: self.
	
	aVisitor visitVariable: variable.
	featureModel accept: aVisitor.! !

!FMAssignment methodsFor: 'accessing' stamp: 'AlexandreBergel 7/1/2012 15:01'!
featureModel
	^ featureModel! !

!FMAssignment methodsFor: 'accessing' stamp: 'AlexandreBergel 7/1/2012 15:01'!
featureModel: anObject
	featureModel := anObject! !

!FMAssignment methodsFor: 'accessing' stamp: 'AlexandreBergel 7/1/2012 15:01'!
variable
	^ variable! !

!FMAssignment methodsFor: 'accessing' stamp: 'AlexandreBergel 7/1/2012 15:01'!
variable: anObject
	variable := anObject! !

!MooseModel methodsFor: '*familiar' stamp: 'AlexandreBergel 7/2/2012 00:27'!
allAssignments
	<navigation: 'All assigment'>
	^ self allWithType: FMAssignment ! !

!MooseModel methodsFor: '*familiar' stamp: 'AlexandreBergel 7/2/2012 00:27'!
allFeatureModels
	<navigation: 'All feature models'>
	^ self allWithType: FMFeatureModel ! !

Object subclass: #FMParser
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Familiar'!

FMParser subclass: #FMASTBuilder
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Familiar'!

!FMASTBuilder methodsFor: 'public' stamp: 'AlexandreBergel 7/1/2012 10:12'!
constraint
	| term prim |
	
	term := PPDelegateParser new.
	prim := PPDelegateParser new.
	
	term setParser: ((prim trim,  self constraintOperator , prim) ==> [ :n |  n second new left: n first ; right: n third; yourself  ] )/ 
					prim.
	prim setParser: 
		(self id trim ==> [ :n | FMConstraintValue new id: n trimBoth ]) /
		(($( asParser trim , term , $) asParser trim) /
					($!! asParser trim, term) ==> [ :n | FMConstraintNegated new expr: n second ])
				.
					
	^ term! !

!FMASTBuilder methodsFor: 'public' stamp: 'AlexandreBergel 7/1/2012 10:01'!
constraintOperator
	^ ( '|'  asParser trim / 
						'&' asParser trim / 
						'->' asParser trim  /  
						'<->' asParser trim) ==> [ :n | FMConstraintBinOp for: n ] ! !

!FMASTBuilder methodsFor: 'private' stamp: 'AlexandreBergel 7/1/2012 15:24'!
featureModel
	^ super featureModel ==> [ :node | 
		| prods cons |
		prods := node third collect: #first. 
		cons := node fourth reject: [ :n | n = $; ]. 
		
		FMFeatureModel new 
			addAllExpressions: prods;
			addAllConstraints: cons;
			yourself ]! !

!FMASTBuilder methodsFor: 'private' stamp: 'AlexandreBergel 7/1/2012 10:38'!
mandatory
	^ super mandatory ==> [ :node | FMFeatureMentatory new id: node ]! !

!FMASTBuilder methodsFor: 'private' stamp: 'AlexandreBergel 7/1/2012 15:14'!
production
	^ super production ==> [ :nodes | 
		FMFeatureProduction new 
			root: nodes first trimBoth ; 
			addAll: nodes third ; 
			yourself ]
! !

!FMParser methodsFor: 'private' stamp: 'AlexandreBergel 7/1/2012 02:13'!
child
	^ self mandatory / self orgroup / self optional / self xorgroup / self mutexgroup! !

!FMParser methodsFor: 'private' stamp: 'AlexandreBergel 7/1/2012 10:56'!
children
	^ self child plus! !

!FMParser methodsFor: 'public' stamp: 'AlexandreBergel 7/1/2012 09:38'!
constraint
	| term prim |
	
	term := PPDelegateParser new.
	prim := PPDelegateParser new.
	
	term setParser: ( prim,  ( '|'  asParser trim / '&' asParser trim / '->' asParser trim  /  '<->' asParser trim ) , prim) / 
					prim.
	prim setParser: ($( asParser trim , term , $) asParser trim) /
					($!! asParser trim, term) /
				self id.
					
	^ term! !

!FMParser methodsFor: 'public' stamp: 'AlexandreBergel 7/1/2012 15:22'!
featureModel
	
	^ 'FM' asParser trim , self leftPar ,  ((self production  , self semiColumn ) plus) ,  (self constraint  , self semiColumn ) star , self rightPar
! !

!FMParser methodsFor: 'private' stamp: 'AlexandreBergel 7/1/2012 14:14'!
fmlIdentifier
	^ (#letter asParser , ( #letter asParser / #digit asParser ) star) flatten trim! !

!FMParser methodsFor: 'private' stamp: 'AlexandreBergel 6/30/2012 14:11'!
id
	^ self fmlIdentifier! !

!FMParser methodsFor: 'private' stamp: 'AlexandreBergel 6/30/2012 22:13'!
leftPar 
	^ $( asParser trim! !

!FMParser methodsFor: 'private' stamp: 'AlexandreBergel 6/30/2012 07:45'!
mandatory
	"Mandatory   : name=ID ;
	ID : ('^')?('a'..'z'|'A'..'Z'|'_') ('a'..'z'|'A'..'Z'|'_'|'0'..'9')*; "
	
	"It seems to be equivalent to fmlIdentifier"
	^ self fmlIdentifier ! !

!FMParser methodsFor: 'private' stamp: 'AlexandreBergel 6/30/2012 14:12'!
mutexgroup
	"Mutexgroup   : LEFT_PAREN features+=ID  ('|' features+=ID)+ RIGHT_PAREN '?' ;"

	^ $( asParser , self id , ( $| asParser , self id ) plus ,  $) asParser , $? asParser! !

!FMParser methodsFor: 'private' stamp: 'AlexandreBergel 7/1/2012 01:59'!
optional
	^ '[' asParser trim , self fmlIdentifier , ']' asParser trim! !

!FMParser methodsFor: 'private' stamp: 'AlexandreBergel 7/1/2012 02:00'!
orgroup
	"Orgroup    : LEFT_PAREN features+=ID  ('|' features+=ID)+ RIGHT_PAREN '+' ;"

	^ self leftPar  , self id , ( self verticalBar  , self id ) plus ,  self rightPar  , $+ asParser trim! !

!FMParser methodsFor: 'private' stamp: 'AlexandreBergel 7/1/2012 10:56'!
production
	^ (self fmlIdentifier ,  ':' asParser trim , self children) trim! !

!FMParser methodsFor: 'public' stamp: 'AlexandreBergel 7/1/2012 15:06'!
program
	^ self statement star ==> [ :node | FMProgram new addAll: node ]! !

!FMParser methodsFor: 'private' stamp: 'AlexandreBergel 6/30/2012 22:14'!
rightPar 
	^ $) asParser trim! !

!FMParser methodsFor: 'private' stamp: 'AlexandreBergel 7/1/2012 01:59'!
semiColumn
	^ $; asParser trim! !

!FMParser methodsFor: 'public' stamp: 'AlexandreBergel 7/1/2012 15:05'!
statement
	^ (self id , $= asParser trim, self featureModel) ==>
			 [ :node | FMAssignment new variable: node first; featureModel: node third ]! !

!FMParser methodsFor: 'private' stamp: 'AlexandreBergel 7/1/2012 01:58'!
verticalBar 
	^ $| asParser trim! !

!FMParser methodsFor: 'private' stamp: 'AlexandreBergel 7/1/2012 01:58'!
xorgroup
	"Xorgroup   : LEFT_PAREN features+=ID  ('|' features+=ID)+ RIGHT_PAREN ;"

	^ self leftPar , self id , ( self verticalBar , self id ) plus ,  self rightPar! !

Object subclass: #FMVisitor
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Familiar'!

FMVisitor subclass: #CollectingVisitor
	instanceVariableNames: 'elements'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Familiar'!

!CollectingVisitor methodsFor: 'as yet unclassified' stamp: 'AlexandreBergel 7/2/2012 00:23'!
elements
	^ elements ! !

!CollectingVisitor methodsFor: 'as yet unclassified' stamp: 'AlexandreBergel 7/2/2012 00:22'!
initialize 
	super initialize.
	elements := OrderedCollection new ! !

!CollectingVisitor methodsFor: 'as yet unclassified' stamp: 'AlexandreBergel 7/2/2012 00:21'!
visitAssigment: aFMAssignment 
	elements add: aFMAssignment ! !

!CollectingVisitor methodsFor: 'as yet unclassified' stamp: 'AlexandreBergel 7/2/2012 00:21'!
visitFeatureModel: aFMFeatureModel 
	elements add: aFMFeatureModel ! !

!CollectingVisitor methodsFor: 'as yet unclassified' stamp: 'AlexandreBergel 7/2/2012 00:21'!
visitProgram: aFMProgram 
	elements add: aFMProgram ! !

!FMVisitor methodsFor: 'user interface' stamp: 'AlexandreBergel 7/2/2012 00:15'!
visitAssigment: aFMAssignment 
	! !

!FMVisitor methodsFor: 'visiting' stamp: 'AlexandreBergel 7/2/2012 00:17'!
visitFeatureModel: aFMFeatureModel 
	! !

!FMVisitor methodsFor: 'visiting' stamp: 'AlexandreBergel 7/2/2012 00:14'!
visitProgram: aFMProgram 
	! !

!FMVisitor methodsFor: 'visiting' stamp: 'AlexandreBergel 7/2/2012 00:16'!
visitVariable: aVariable 
	! !
